using System;
using System.Linq;
using System.Runtime.InteropServices;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint.WebPartPages;

namespace RH.Sharepoint.Shareconnect
{
	[Guid("2355ab16-9f32-48ad-aa6d-5879b735726b")]
	public class Sample : Microsoft.SharePoint.WebPartPages.WebPart
	{
		private bool _Error;
		private string _Property;
		private SPList _SiteUsers;

		[Personalizable(PersonalizationScope.Shared)]
		[WebBrowsable(true)]
		[System.ComponentModel.Category("My Property Group")]
		[WebDisplayName("Property")]
		[WebDescription("Meaningless Property")]
		public string Property
		{
			get
			{
				if (_Property == null)
				{
					_Property = "Hello SharePoint";
				}
				return _Property;
			}
			set { _Property = value; }
		}

		public override ToolPart[] GetToolParts()
		{
			try
			{
				var toolparts = new ToolPart[2];
				var wptp = new WebPartToolPart();
				var custom = new CustomPropertyToolPart();
				toolparts[0] = custom;
				toolparts[1] = wptp;

				// This is the custom ToolPart.
				toolparts[0] = new CustomToolPart();

				return toolparts;
			}
			catch (Exception ex)
			{
				HandleException(new Exception(string.Format("Error in ToolPart: {0}", Helper.GetExeption(ex))));
			}
			return null;
		}

		public Sample()
		{
			ExportMode = WebPartExportMode.All;
		}

		/// <summary>
		/// Create all your controls here for rendering.
		/// Try to avoid using the RenderWebPart() method.
		/// </summary>
		protected override void CreateChildControls()
		{
			if (_Error) return;
			
			try
			{
				base.CreateChildControls();

				// Your code here...
				Controls.Add(new LiteralControl(Property));

				RenderSiteUsersAsHtml();

				GetSiteUsersList();
				GetUserForm();

				CreateWelcomeText();
			}
			catch (Exception ex)
			{
				HandleException(ex);
			}
		}

		/// <summary>
		/// get the Site Users List with elevated privileges
		/// </summary>
		private void GetSiteUsersList()
		{
			Helper.WriteUlsVerbose("Connecting to Site Users Liste");

			using (var elevatedSite = new SPSite(SPContext.Current.Web.Url, SPContext.Current.Site.SystemAccount.UserToken))
			using (var elevatedWeb = elevatedSite.RootWeb)
			{
				if (elevatedWeb.GetList("User Information List", out _SiteUsers) == false)
				{
					Helper.WriteUlsError("Site Users Liste nicht gefunden");
					throw new ArgumentException("Site Users Liste nicht gefunden.<br/>");
				}
			}
		}

		private void GetUserForm()
		{
			// only add the newForm, if the user may edit items
			if (_SiteUsers == null || !_SiteUsers.DoesUserHavePermissions(SPBasePermissions.EditListItems))
			{
				return;
			}

			// connect to the rootweb, to read the users list
			var rootWeb = SPContext.Current.Site.RootWeb;
			// get the current user and corresponding item in the user information list
			SPUser user = SPContext.Current.Web.CurrentUser;
			SPList list = rootWeb.Lists["User Information List"];
			SPListItem userItem = list.GetItemById(user.ID);

			// set the context, so the controls will work
			SPContext context = SPContext.GetContext(HttpContext.Current, userItem.ID, list.ID, rootWeb);

			// container for the controls
			var content = new HtmlGenericControl("div");
			content.AddLineBreak(1);
			content.Controls.Add(new Label
			{
				Text = "Edit current User",
				CssClass = "ms-sitetitle"
			});
			content.AddLineBreak(2);

			// Username
			content.Controls.Add(new Label { Text = "Username:" });
			content.AddLineBreak(1);
			var username = new TextField
			{
				ID = "Username",
				FieldName = "Title",
				ItemId = userItem.ID,
				ListId = list.ID,
				ControlMode = SPControlMode.Edit,
				RenderContext = context,
				ItemContext = context
			};
			content.Controls.Add(username);
			content.AddLineBreak(1);

			// about me
			content.Controls.Add(new Label { Text = "About me:" });
			content.AddLineBreak(1);
			var aboutMe = new RichTextField
			{
				ID = "AboutMe",
				FieldName = "Notes",
				ItemId = userItem.ID,
				ListId = list.ID,
				ControlMode = SPControlMode.Edit,
				RenderContext = context,
				ItemContext = context
			};
			content.Controls.Add(aboutMe);
			content.AddLineBreak(1);

			var updateButton = new Button
			{
				ID = "UpdateUserButton",
				Text = "Update User"
			};
			updateButton.Click += UpdateButtonClick;
			content.Controls.Add(updateButton);

			Controls.Add(content);
		}

		void UpdateButtonClick(object sender, EventArgs e)
		{
			try
			{
				var rootWeb = SPContext.Current.Site.RootWeb;
				// get the current user and corresponding item in the user information list
				SPUser user = SPContext.Current.Web.CurrentUser;
				SPList list = rootWeb.Lists["User Information List"];
				//SPListItem userItem = list.GetItemById(user.ID);
				// get value from controls
				var usernameField = (TextField)FindControl("Username");
				var aboutMeField = (NoteField)FindControl("AboutMe");

				var item = list.GetItemById(user.ID);
				item[SPBuiltInFieldId.Title] = usernameField.Value;
				item[SPBuiltInFieldId.Notes] = aboutMeField.Value;
				if (item.RequiredFieldsSet())
				{
					item.Update();
					// after the item has been saved, it will not be updated in the previously added control
					// the page needs to be reloaded
					return;
				}
				Controls.Add(new Label
				{
					Text = "Bitte alle Felder ausfllen",
					CssClass = "ms-error"
				});
			}
			catch (Exception ex)
			{
				HandleException(ex);
			}
		}

		private void RenderSiteUsersAsHtml()
		{
			SPSite site = SPContext.Current.Site;
			var rootWeb = site.RootWeb;
			SPList list = rootWeb.Lists.Cast<SPList>().SingleOrDefault(l => l.Title == "User Information List");
			SPView view = list.DefaultView;

			Controls.Add(new LiteralControl(view.RenderAsHtml()));
		}

		/// <summary>
		/// Ensures that the CreateChildControls() is called before events.
		/// Use CreateChildControls() to create your controls.
		/// </summary>
		/// <param name="e"></param>
		protected override void OnLoad(EventArgs e)
		{
			if (_Error) return;
			
			try
			{
				base.OnLoad(e);
				EnsureChildControls();

				// Your code here...
			}
			catch (Exception ex)
			{
				HandleException(ex);
			}
		}

		/// <summary>
		/// Clear all child controls and add an error message for display.
		/// </summary>
		/// <param name="ex"></param>
		private void HandleException(Exception ex)
		{
			_Error = true;
			Controls.Clear();
			Controls.Add(new LiteralControl(string.Format("{0}<br/>{1}", ex.Message, ex)));
		}

		/// <summary>
		/// Show localized Welcome text
		/// </summary>
		private void CreateWelcomeText()
		{
			uint lcid = SPContext.Current.RegionalSettings.LocaleId;
			string welcome = SPUtility.GetLocalizedString("$Resources:Welcome", "RHShareconnect", lcid);

			if (SPContext.Current.Web.CurrentUser != null)
			{
				welcome += string.Format(" {0}", SPContext.Current.Web.CurrentUser.Name);
			}

			Controls.Add(new LiteralControl(welcome));
			this.AddLineBreak(1);
		}
	}
}
